package de.mbutscher.wikiandpad;

import java.util.ArrayList;
import java.util.List;

import de.mbutscher.wikiandpad.alphabeta.R;
import de.mbutscher.wikiandpad.db.MatchTermEntry;
import de.mbutscher.wikiandpad.parsing.NonTerminalNode;
import de.mbutscher.wikiandpad.parsing.SyntaxNode;
import de.mbutscher.wikiandpad.utils.MiscEvent;
import de.mbutscher.wikiandpad.utils.MiscEventSource;
import de.mbutscher.wikiandpad.utils.ui.FragmentGuiHelper;
import android.os.Bundle;
import android.support.v4.app.ListFragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;


public class PageStructureFragment extends ListFragment
        implements MiscEventSource {
//    protected ArrayAdapter<String> arrAdapt;
    
    protected FragmentGuiHelper<WikiWordEditActivity,
            PageStructureFragment> guiHelper;
    
    public static class AdapterEntry extends MatchTermEntry {
        
        protected String title;
        
        public AdapterEntry(int firstcharpos, int charlength, String title) {
            super(firstcharpos, charlength);
            this.title = title;
        }
        
        @Override
        public String toString() {
            return title;
        }
    }
    

    protected List<AdapterEntry> listContent = null;

    protected ArrayAdapter<AdapterEntry> EMPTY_ARR_ADAPTER;

    public void updateList() {
        
        WikiPage wikiPage = getEditor().getCurrentWikiPage();
        
        if (wikiPage == null) {
            populateList(null, true);
            return;
        }
        
        NonTerminalNode syntaxNode = (NonTerminalNode) wikiPage.getLivePageAst();
        
        if (syntaxNode == null) {
            populateList(null, true);
            return;
        }
        
        
        ArrayList<AdapterEntry> entries = new ArrayList<AdapterEntry> ();

        for (SyntaxNode heading: syntaxNode.iterDeepByName("heading", 0)) {
            entries.add(new AdapterEntry(heading.getPos(), 0,
                    ((SyntaxNode) heading.dGet("contentNode")).getString()));
        }
        
        
        populateList(entries, true);
    }

    
    public synchronized void populateList(List<AdapterEntry> listContent,
            boolean keepListState) {
        
        this.listContent = listContent;

        ArrayAdapter<AdapterEntry> arrAdapt;

        TextView tvSearching = guiHelper.findViewById(R.id.tvSearching);

        // getListView().setVisibility(View.GONE);
        tvSearching.setVisibility(View.GONE);

        if (listContent == null || listContent.size() == 0) {
            arrAdapt = EMPTY_ARR_ADAPTER;
        }
        else
        {
            arrAdapt = new ArrayAdapter<AdapterEntry>(
                    guiHelper.getActivity(), R.layout.simple_list_item,
                    listContent);
        }

        getListView().setAdapter(arrAdapt);
        
        /*
        if (lastListState != null) {
            getListView().onRestoreInstanceState(lastListState);
            lastListState = null;
        }
        */
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        guiHelper = new FragmentGuiHelper<WikiWordEditActivity,
                PageStructureFragment>(this);

        // Initiate quasi-constant (can't be done earlier)
        EMPTY_ARR_ADAPTER = new ArrayAdapter<AdapterEntry>(
                guiHelper.getActivity(),
                R.layout.simple_list_item, new ArrayList<AdapterEntry>());
    }

    
    @Override
    public View onCreateView (LayoutInflater inflater,
            ViewGroup container, Bundle savedInstanceState) {

        guiHelper.setContentView(R.layout.page_structure_list, inflater,
                container, savedInstanceState);

        registerForContextMenu(guiHelper.getListView());

        return guiHelper.getContentView();
    }
    
    /*
    protected void updateShowHideUi() {
        Button btnOpenWiki = guiHelper.findViewById(R.id.btnOpenWiki);
        Button btnCreateWord = guiHelper.findViewById(R.id.btnCreateWord);

        WikiDocument wikiDoc = WikiDocument.getActiveDocument();
                
        if (wikiDoc == null) {
            // No wiki open
            btnOpenWiki.setVisibility(View.VISIBLE);
            btnCreateWord.setVisibility(View.GONE);
            tfSearchWikiWord.setVisibility(View.GONE);
        }
        else {
            // Wiki open
            btnOpenWiki.setVisibility(View.GONE);
            btnCreateWord.setVisibility(View.VISIBLE);
            tfSearchWikiWord.setVisibility(View.VISIBLE);
        }
    }
    */
    
    /*
    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        
        WikiDocument wikiDoc = WikiDocument.getActiveDocument();
        if (wikiDoc == null)
            return;
        
        outState.putString("lastSearch", lastSearch);
        outState.putLong("lastDatabaseChangeCount", lastDatabaseChangeCount);
        outState.putString("wikiConfigPath", wikiDoc.getWikiConfigPath());
        outState.putParcelable("listState", getListView().onSaveInstanceState());
    }
    */


    @Override
    public void onPause() {
        super.onPause();
        
        // listUpdate.setThread(null);
    }


    @Override
    public void onResume() {
        super.onResume();
        
        // updateShowHideUi(); 
        
        /*String WikiWordCcPAT = "(?:[" +
                StringOps.UPPERCASE +
                "]+[" +
                StringOps.LOWERCASE +
                "]+[" +
                StringOps.UPPERCASE +
                "]+[" +
                StringOps.LETTERS + StringOps.DIGITS +
                "]*|[" +
                StringOps.UPPERCASE +
                "]{2,}[" +
                StringOps.LOWERCASE +
                "]+)";

        Pattern re = java.util.regex.Pattern.compile("\\b(?<!~)" + WikiWordCcPAT + "\\b", 0);
        Matcher mt = re.matcher("hjdghf AutoCompletion");

        // Log.d("WikiAndPad", java.util.regex.Pattern.quote("["));
        // Log.d("WikiAndPad", Boolean.toString(mt.find()));
        // Log.d("WikiAndPad", mt.find()); */

        
        updateList();
    }
    
    /*
    public void onDetach() {
        Fragment f = getActivity().getSupportFragmentManager()
                .findFragmentByTag("baseTest");

        Log.d("WikiAndPad", "--Fragment.onDetach" + StringOps.toSaveString(f));
        
        if (f != null) {
            getActivity().getSupportFragmentManager().beginTransaction().remove(
                    f).commitAllowingStateLoss();
            getActivity().getSupportFragmentManager().executePendingTransactions();
        }
        super.onDetach();
    }
    */

    
    /*
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = guiHelper.getActivity().getMenuInflater();
        inflater.inflate(R.menu.wiki_word_list, menu);
        return true;
    }
    
    @Override
    public void onPrepareOptionsMenu (Menu menu) {
        MenuItem item = menu.findItem(R.id.mnCloseWiki);
        item.setVisible(WikiDocument.getActiveDocument() != null);

        item = menu.findItem(R.id.mnSearch);
        item.setVisible(WikiDocument.getActiveDocument() != null);
    }
    */
    
    /*
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Intent intent;

        switch (item.getItemId()) {
        case R.id.mnOpenWiki:
            FolderPicker mFileDialog = new FolderPicker(
                    guiHelper.getActivity(), onOpenWiki,
                    android.R.style.Theme, true);
            mFileDialog.show();

            return true;
        case R.id.mnCloseWiki:
            try {
                WikiDocument.openDocument(null, true);
            }
            catch (FileNotFoundException e) {}
            catch (IOException e) {}
            catch (WikiDataException e) {}

            updateShowHideUi();
            updateList();
            return true;
        case R.id.mnPreferences:
            intent = new Intent(guiHelper.getActivity().getBaseContext(),
                    Preferences.class);
            startActivity(intent);
            
            return false;
        case R.id.mnSearch:
            guiHelper.getActivity().onSearchRequested();
            
            return false;
        case R.id.mnAbout:
            intent = new Intent(Intent.ACTION_VIEW);
            intent.setComponent(new ComponentName(guiHelper.getActivity(),
                    AboutActivity.class));

            startActivity(intent);
            
            return false;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    */
    
    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        getEditor().setSelection(
                (AdapterEntry) l.getAdapter().getItem(position),
                ScrollingEditText.ScrollTo.TOP);

        getFragmentManager().popBackStack();
        
        /*
        getMiscEvent().cloneIt().put("request openWikiWord")
                .put("wikiPageName", listContent.get(position).wikiPageName)
                .processSend();
        */
    }
    
    /*
    public void onCmdClearSearch() {
        tfSearchWikiWord.setText("");
    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View v,
            ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        if (v == getListView()) {
            MenuInflater inflater = guiHelper.getActivity().getMenuInflater();
            inflater.inflate(R.menu.wiki_word_listitem_context, menu);
        }
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterContextMenuInfo info =
                (AdapterContextMenuInfo) item.getMenuInfo();
        MatchTermEntry mte;
        switch (item.getItemId()) {
        case R.id.mnRenameThis:
            mte = listContent.get(info.position);
            if (WikiDocument.getActiveDocument().getWikiName().equals(
                    mte.wikiPageName)) {
                guiHelper.alertInfo("Rename of root page not supported");
            }
            else {
                onRename(mte.wikiPageName);
            }
            
            return false;

        case R.id.mnDeleteThis:
            mte = listContent.get(info.position);
            if (WikiDocument.getActiveDocument().getWikiName().equals(
                    mte.wikiPageName)) {
                guiHelper.alertInfo("Delete of root page not possible");
            }
            else {
                onDelete(mte.wikiPageName);
            }

            return false;
        default:
            return super.onContextItemSelected(item);
        }
    }

    */

    protected WikiWordEditFragment getEditor() {
        return (WikiWordEditFragment) getFragmentManager().findFragmentByTag(
                getArguments().getString("editorFragmentKey"));
    }


    protected MiscEvent miscEvent = new MiscEvent(this);
    
    public MiscEvent getMiscEvent() {
        return miscEvent;
    }
    
}
