package de.mbutscher.wikiandpad;

import java.util.ArrayList;
import java.util.List;

import de.mbutscher.wikiandpad.alphabeta.R;
import de.mbutscher.wikiandpad.db.DbReadAccessException;
import de.mbutscher.wikiandpad.db.MatchTermEntry;
import de.mbutscher.wikiandpad.utils.NotCurrentThreadException;
import de.mbutscher.wikiandpad.utils.StringOps;
import de.mbutscher.wikiandpad.utils.ThreadHolder;
import de.mbutscher.wikiandpad.utils.ui.ActivityGuiHelper;
import android.app.ListActivity;
import android.app.SearchManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.graphics.Typeface;
import android.os.Bundle;
import android.os.Parcelable;
import android.text.Spannable;
import android.text.style.StyleSpan;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public class SearchableActivity extends ListActivity {

    protected ActivityGuiHelper<SearchableActivity> guiHelper;
    protected ThreadHolder listUpdate = new ThreadHolder();
    protected List<ListEntry> listContent = null;
    // protected long lastDatabaseChangeCount;
    // protected Parcelable lastListState = null;



    protected SearchListAdapter EMPTY_ARR_ADAPTER;
    protected SearchAndReplaceOperation sarOp;

    
    protected static class ListEntry {
        public MatchTermEntry target;
        public String title;
        public String before;
        public String highlight;
        public String after;

        public ListEntry(MatchTermEntry target, String title, String before,
                String highlight, String after) {
            super();
            this.target = target;
            this.title = title;
            this.before = before;
            this.highlight = highlight;
            this.after = after;
        }
    }
    
    protected static class SearchListAdapter extends ArrayAdapter<ListEntry> {
        public SearchListAdapter(Context context, int textViewResourceId) {
            super(context, textViewResourceId);
        }

        public SearchListAdapter(Context context, int textViewResourceId,
                List<ListEntry> objects) {
            super(context, textViewResourceId, objects);
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            // TODO: Reuse existing view

            View result = View.inflate(getContext(),
                    R.layout.big_little_list_item, null); // parent);
            
            ListEntry entry = getItem(position);
            
            TextView tv = (TextView) result.findViewById(R.id.text1);
            tv.setText(entry.title);
            
            tv = (TextView) result.findViewById(R.id.text2);
            tv.setText(entry.before + entry.highlight + entry.after);
            
            ((Spannable)tv.getText()).setSpan(new StyleSpan(Typeface.BOLD),
                    entry.before.length(),
                    entry.before.length() + entry.highlight.length(),
                    Spannable.SPAN_EXCLUSIVE_INCLUSIVE);

            return result;
        }
    }

    /**
     * Shows "searching" instead of empty list until next call to
     */
    public synchronized void showSearching() {
        // getListView().setVisibility(View.GONE);
        TextView tvSearching = (TextView) findViewById(R.id.tvSearching);
        tvSearching.setVisibility(View.VISIBLE);
    }

    
    
    public synchronized void populateList(List<ListEntry> listContent) {

        this.listContent = listContent;

        SearchListAdapter arrAdapt;
        TextView tvSearching = (TextView) findViewById(R.id.tvSearching);

        tvSearching.setVisibility(View.GONE);

        if (listContent == null || listContent.size() == 0) {
            arrAdapt = EMPTY_ARR_ADAPTER;
        }
        else {
            arrAdapt = new SearchListAdapter(this,
                    R.layout.big_little_list_item, listContent);
        }
        
        ListView lv = getListView();
        Parcelable p = lv.onSaveInstanceState();
        lv.setAdapter(arrAdapt);
        lv.onRestoreInstanceState(p);

        
        /*if (lastListState != null) {
            getListView().onRestoreInstanceState(lastListState);
            lastListState = null;
        }*/
    }

    
    /**
     * Initiates an update of the list in the background
     */
    public synchronized void updateList() {
        final WikiDocument wikiDoc = WikiDocument.getActiveDocument();

        if (wikiDoc == null)
        {
            listUpdate.setThread(null);
            populateList(null);
            return;
        }

        /* final boolean keepListState;
        
        if (wikiDoc.isSameDatabaseChangeCount(lastDatabaseChangeCount)) {
            if (listContent == null) {
                // Search parameters didn't change but we have lost the
                // list contents -> rebuild but restore list state afterwards
                keepListState = true;
            }
            else
                // No changes
                return;
        }
        else
            keepListState = false;
        */



        listUpdate.prepare(new Runnable() {
            public void run() {
                try {
                    List<MatchTermEntry> table = wikiDoc.searchWiki(sarOp);
                    if (!listUpdate.isValidThread())
                        return;
                    
                    final List<ListEntry> result = new ArrayList<ListEntry>(
                            table.size());
                    
    
                    for (MatchTermEntry mte: table) {
                        if (!listUpdate.isValidThread())
                            return;
    
                        int beforePos = mte.firstcharpos - 20;
                        beforePos = beforePos >= 0 ? beforePos : 0;
                        
                        WikiPage page = wikiDoc.getWikiPage(mte.wikiPageName);
                        String content = page.getLiveText();
                        if (StringOps.isEmpty(content))
                            continue;
                        
                        int lastPos = mte.firstcharpos + mte.charlength;
    
                        int afterPos = lastPos + 30;
                        afterPos = afterPos < content.length() ? afterPos :
                                content.length() - 1;
                        
                        ListEntry le = new ListEntry(mte, page.getWikiPageName(),
                                content.substring(beforePos, mte.firstcharpos),
                                content.substring(mte.firstcharpos, lastPos),
                                content.substring(lastPos, afterPos));
                        
                        result.add(le);
                    }
    
                    if (!listUpdate.isValidThread())
                        return;
                    
                    final Thread listThread = Thread.currentThread();
    
                    runOnUiThread(new Runnable() {
                        public void run() {
                            if (!listUpdate.isValidThread(listThread))
                                return;
    
                            /*lastDatabaseChangeCount =
                                    wikiDoc.getDatabaseChangeCount();*/
    
                            populateList(result);
                        }
                    });
                }
                catch (NotCurrentThreadException e) {
                    return;
                }
                catch (DbReadAccessException e) {
                    return;  // TODO: Error message
                }
            }
        });

        showSearching();
        listUpdate.start();

        // populateList(wikiDoc.getWikiWordMatchTermsWith(
        //         tfSearchWikiWord.getText().toString(), "", false));
    }

    
    /*@Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        
        WikiDocument wikiDoc = WikiDocument.getActiveDocument();
        if (wikiDoc == null)
            return;

        outState.putLong("lastDatabaseChangeCount", lastDatabaseChangeCount);
        outState.putString("wikiConfigPath", wikiDoc.getWikiConfigPath());
        outState.putParcelable("listState", getListView().onSaveInstanceState());
    }*/


    @Override
    protected void onPause() {
        super.onPause();
        listUpdate.setThread(null);
    }


    @Override
    protected void onResume() {
        super.onResume();
        
        updateList();
    }

    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        WikiDocument wikiDoc = WikiDocument.getActiveDocument();
        
        Intent intent = getIntent();
        
        if (wikiDoc == null) {
            Toast.makeText(this,
                    String.format("No open wiki"),
                    Toast.LENGTH_SHORT).show();
            finish();
            return;
        }

        /*if (savedInstanceState != null && wikiDoc.getWikiConfigPath().equals(
                savedInstanceState.getString("wikiConfigPath"))) {
            lastDatabaseChangeCount = savedInstanceState.getLong(
                    "lastDatabaseChangeCount");
            lastListState = savedInstanceState.getParcelable("listState");
        }*/


        String searchStr = intent.getStringExtra(SearchManager.QUERY);
        
        if (!StringOps.notEmpty(searchStr)) {
            Toast.makeText(this,
                    String.format("Invalid search"),
                    Toast.LENGTH_SHORT).show();
            finish();
            return;
        }
        
        sarOp = new SearchAndReplaceOperation();
        sarOp.searchStr = searchStr;
        sarOp.prepare();

 
/*
        if (wikiDoc == null || savedInstanceState == null ||
                !savedInstanceState.containsKey("lastDatabaseChangeCount")) {
            lastSearch = null;
        }
        else {
            // First check if same wiki open yet
            if (wikiDoc.getWikiConfigPath().equals(savedInstanceState
                    .getString("wikiConfigPath"))) {
                lastSearch = savedInstanceState.getString("lastSearch");
                lastDatabaseChangeCount = savedInstanceState.getLong(
                        "lastDatabaseChangeCount");
                lastListState = savedInstanceState.getParcelable("listState");
            }
            else
                lastSearch = null;
        }
        
*/

        // Initiate quasi-constant (can't be done earlier)
        EMPTY_ARR_ADAPTER = new SearchListAdapter(this,
                R.layout.big_little_list_item, new ArrayList<ListEntry>());

        
        setContentView(R.layout.searchable_list);
        
        guiHelper = new ActivityGuiHelper<SearchableActivity>(this);
        
        guiHelper.setOnClickListener(R.id.btnBack, "onCmdBack");

        //registerForContextMenu(getListView());
    }
    
    /* @Override
    public synchronized void onConfigurationChanged(Configuration newConfig) {
        Parcelable listState = getListView().onSaveInstanceState();
        ListAdapter adapt = getListView().getAdapter();
        TextView tvSearching = (TextView) findViewById(R.id.tvSearching);
        int tvSearchVis = tvSearching.getVisibility();
        
        super.onConfigurationChanged(newConfig);

        // setContentView(R.layout.searchable_list);
        //populateList(listContent, true);
        
        getListView().setAdapter(adapt);
        getListView().onRestoreInstanceState(listState);
        tvSearching.setVisibility(tvSearchVis);
    }*/

    
    @Override
    protected void onListItemClick(ListView l, View v, int position, long id) {
        Intent intent = new Intent(Intent.ACTION_EDIT);
        intent.setComponent(new ComponentName(this, WikiWordEditActivity.class));
        intent.putExtra("wikiConfigPath", WikiDocument.getActiveDocument()
                .getWikiConfigPath());
        intent.putExtra("wikiPageName",
                listContent.get(position).target.wikiPageName);
    
        intent.putExtra("matchTermEntryBundle",
                listContent.get(position).target.toBundle());
        startActivity(intent);
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.wiki_search, menu);
        return true;
    }
    
    @Override
    public boolean onPrepareOptionsMenu (Menu menu) {
        MenuItem item = menu.findItem(R.id.mnSearch);
        item.setVisible(WikiDocument.getActiveDocument() != null);
        
        return true;
    }

    
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case R.id.mnPreferences:
            Intent intent = new Intent(getBaseContext(), Preferences.class);
            startActivity(intent);
            
            return false;
        case R.id.mnSearch:
            onSearchRequested();
            
            return false;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    
    public void onCmdBack() {
        finish();
    }

}
