package de.mbutscher.wikiandpad;

import java.lang.Thread.UncaughtExceptionHandler;

import de.mbutscher.wikiandpad.utils.MiscEvent;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;

public class WikiAndPadApplication extends Application {

    protected static WikiAndPadApplication theApplication;
    
    protected ClipboardCatcher clipboardCatcher;
    
    protected MiscEvent mainMiscEvent;

    @Override
    public void onCreate() {
        super.onCreate();
        WikiAndPadApplication.theApplication = this;
        mainMiscEvent = new MiscEvent();

        // Install exception handler (complicated way to catch possible
        //   exception during retrieving of preference)

        UncaughtExceptionHandler uceh =
                Thread.getDefaultUncaughtExceptionHandler();
        
        Thread.setDefaultUncaughtExceptionHandler(new TheExceptionHandler(this,
                Thread.getDefaultUncaughtExceptionHandler()));

        SharedPreferences pref = WikiAndPadApplication.getAppPrefs();
        if (!pref.getBoolean("debug_writeToExternalLog", false))
            Thread.setDefaultUncaughtExceptionHandler(uceh);

        clipboardCatcher = new ClipboardCatcher(this);
    }

    public static WikiAndPadApplication getApp() {
        return theApplication;
    }

    public static SharedPreferences getAppPrefs() {
        return theApplication.getSharedPreferences("wikidpad",
                Context.MODE_PRIVATE);
    }

    public ClipboardCatcher getClipboardCatcher() {
        return clipboardCatcher;
    }
    
    public void stopClipboardCatcher() {
        clipboardCatcher.stop();
    }
    
    public MiscEvent getMainMiscEvent() {
        return mainMiscEvent;
    }

    public static MiscEvent getMainEvent() {
        return getApp().getMainMiscEvent();
    }
  
    public static MiscEvent cloneMainEvent(Object source) {
        return getMainEvent().cloneIt(source);
    }
}
