package de.mbutscher.wikiandpad;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.folder.FolderPicker;

import de.mbutscher.wikiandpad.alphabeta.R;
import de.mbutscher.wikiandpad.db.DbWriteAccessException;
import de.mbutscher.wikiandpad.db.MatchTermEntry;
import de.mbutscher.wikiandpad.db.WikiDataException;
import de.mbutscher.wikiandpad.utils.MiscEvent;
import de.mbutscher.wikiandpad.utils.MiscEventSource;
import de.mbutscher.wikiandpad.utils.RichMEListener;
import de.mbutscher.wikiandpad.utils.StringOps;
import de.mbutscher.wikiandpad.utils.Stringifier;
import de.mbutscher.wikiandpad.utils.ThreadHolder;
import de.mbutscher.wikiandpad.utils.ui.StringArrayListAdapter;
import de.mbutscher.wikiandpad.utils.ui.FragmentGuiHelper;
import android.app.AlertDialog;
import android.content.ComponentName;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.os.Parcelable;
import android.support.v4.app.ListFragment;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;


public class WikiWordListFragment extends ListFragment
        implements MiscEventSource {
//    protected ArrayAdapter<String> arrAdapt;
    
    protected FragmentGuiHelper<WikiWordListActivity,
            WikiWordListFragment> guiHelper;
    protected ThreadHolder listUpdate = new ThreadHolder();
    
    protected StringArrayListAdapter<MatchTermEntry> listAdapter;
    
    protected String lastSearch = null;
    protected String lastSortOrder = null;
    protected TextView tfSearchWikiWord;
    protected Parcelable lastListState = null;
    protected long lastDatabaseChangeCount;
    
    // protected ArrayListAdapter<String> EMPTY_ARR_ADAPTER;

    protected final android.content.DialogInterface.OnClickListener onOpenWiki =
            new android.content.DialogInterface.OnClickListener() {
        public void onClick(android.content.DialogInterface dialog,
                int which) {
            
            boolean lruModified = false;
            
            try {
                // String rawIni = StringOps.loadEntireTxtFile(
                //         ((FolderPicker) dialog).getPath());

                // Map<String, String> config = StringOps.parseIni(rawIni);
                try {
                    String path = ((FolderPicker) dialog).getPath();
                    path = (new File(path)).getAbsolutePath();
                    
                    SharedPreferences prefs = WikiAndPadApplication.getAppPrefs();
                    
                    ArrayList<String> recentPaths = StringOps.strToStrList(
                            prefs.getString("lruWikiPaths", ""));
                    
                    lruModified = lruModified || recentPaths.remove(path);
                    WikiDocument.openDocument(path);
                    
                    recentPaths.add(0, path);
                    
                    // TODO Make max. number of recent entries configurable
                    while (recentPaths.size() > 20)
                        recentPaths.remove(recentPaths.size() - 1);
                    
                    Editor editor = prefs.edit();
                    editor.putString("lruWikiPaths",
                            StringOps.strListToStr(recentPaths));
                    editor.commit();
                    
                } catch (FileNotFoundException e) {
                    // TheExceptionHandler.logException(e);
                    guiHelper.alertProblem("File %s not found",
                            ((FolderPicker) dialog).getPath());
                } catch (IOException e) {
                    // TheExceptionHandler.logException(e);
                    Toast.makeText(guiHelper.getActivity(),
                            String.format("File %s can't be opened: %s",
                            ((FolderPicker) dialog).getPath(),
                            e.getLocalizedMessage()),
                            Toast.LENGTH_LONG).show();

                } catch (WikiDataException e) {
                    // TheExceptionHandler.logException(e);
                    guiHelper.alertError(e.getLocalizedMessage());
                }

                lastSearch = null;
                updateShowHideUi();
                updateList();

                /*
                ArrayAdapter<String> arrAdapt;
                arrAdapt.clear();
                for(String k: config.keySet()) {
                    arrAdapt.add(k + "=" + config.get(k));
                }
                */
            }
            /*
            catch(FileNotFoundException e) {
            }
            catch (IOException e) {
            }
            */
            finally {}

            // arrAdapt.insert(((FolderPicker) dialog).getPath(), 0);
        }
    };
    
    
    protected final View.OnClickListener onBtnOpenWiki =
            new View.OnClickListener() {

        public void onClick(View v) {
            SharedPreferences prefs = WikiAndPadApplication.getAppPrefs();
            
            ArrayList<String> paths = StringOps.strToStrList(prefs.getString(
                    "lruWikiPaths", ""));

            FolderPicker mFileDialog = new FolderPicker(
                    guiHelper.getActivity(), onOpenWiki,
                    android.R.style.Theme, true, paths);
            mFileDialog.show();
        }
    };

    
    protected final View.OnClickListener onBtnCreateWord =
            new View.OnClickListener() {

        public void onClick(View v) {
            String newWikiPageName = tfSearchWikiWord.getText().toString();
            if (newWikiPageName.length() == 0) {
                Toast.makeText(guiHelper.getActivity(),
                        "Can't create empty word",
                        Toast.LENGTH_SHORT).show();
                return;
            }
            
            /*
            Intent intent = new Intent(Intent.ACTION_EDIT);
            intent.setComponent(new ComponentName(guiHelper.getActivity(),
                    WikiWordEditActivity.class));
            intent.putExtra("wikiPageName", newWikiPageName);
            
            startActivity(intent);
            */
            
            getMiscEvent().cloneIt().put("request openWikiWord")
                    .put("wikiConfigPath",
                        WikiDocument.getActiveDocument().getWikiConfigPath())
                    .put("wikiPageName", newWikiPageName)
                    .send();
        }
    };
    
    /**
     * Shows "searching" instead of empty list until next call to
     */
    public synchronized void showSearching() {
        // getListView().setVisibility(View.GONE);
        TextView tvSearching = guiHelper.findViewById(R.id.tvSearching);
        tvSearching.setVisibility(View.VISIBLE);
    }
    
    public synchronized void populateList(List<MatchTermEntry> listData,
            boolean keepListState) {

//         ArrayListAdapter<String> arrAdapt;

        TextView tvSearching = guiHelper.findViewById(R.id.tvSearching);

        // getListView().setVisibility(View.GONE);
        tvSearching.setVisibility(View.GONE);

        /*
        if (listData == null || listData.size() == 0) {
            // getListView().setVisibility(View.GONE);
        }
        else
        {
            // getListView().setVisibility(View.VISIBLE);
        }
        */
        
        
        if (keepListState)
        {
            ListView lv = getListView();
            Parcelable p = lv.onSaveInstanceState();
            listAdapter.setListData(listData);
            lv.onRestoreInstanceState(p);
        }
        else
            listAdapter.setListData(listData);

        if (lastListState != null) {
            getListView().onRestoreInstanceState(lastListState);
            lastListState = null;
        }
    }

    /**
     * Initiates an update of the list in the background
     */
    public synchronized void updateList() {
        final WikiDocument wikiDoc = WikiDocument.getActiveDocument();
        
        TextView tvWordlistHeader = guiHelper.findViewById(
                R.id.tvWordlistHeader);


        if (wikiDoc == null)
        {
            listUpdate.setThread(null);
            populateList(null, false);
            tvWordlistHeader.setText("Wordlist");
            return;
        }

        tvWordlistHeader.setText("Wordlist of " + wikiDoc.getWikiName());

        final String search = tfSearchWikiWord.getText().toString();
        final String sortOrder = WikiAndPadApplication.getAppPrefs().getString(
                "wikipageList_order", "alphabetical");
        
        final boolean keepListState;
        
        if (search.equals(lastSearch) && sortOrder.equals(lastSortOrder) &&
                wikiDoc.isSameDatabaseChangeCount(lastDatabaseChangeCount)) {
            if (listAdapter == null) {
                // Search parameters didn't change but we have lost the
                // list contents -> rebuild but restore list state afterwards
                keepListState = true;
            }
            else
                // No changes
                return;
        }
        else
            keepListState = true; // false;
        
        listUpdate.prepare(new Runnable() {
            public void run() {
                final List<MatchTermEntry> table = wikiDoc
                        .getWikiWordMatchTermsWith(search, sortOrder, false);

                if (!listUpdate.isValidThread())
                    return;

                /*
                final List<String> words = new ArrayList<String>(table.size());
                for (MatchTermEntry mte: table)
                    words.add(mte.matchterm);
                */

                if (!listUpdate.isValidThread())
                    return;
                
                final Thread listThread = Thread.currentThread();

                guiHelper.getActivity().runOnUiThread(new Runnable() {
                    public void run() {
                        if (!listUpdate.isValidThread(listThread))
                            return;

                        lastSearch = search;
                        lastSortOrder = sortOrder;
                        lastDatabaseChangeCount =
                                wikiDoc.getDatabaseChangeCount();

                        populateList(table, keepListState);
                    }
                });
            }
        });

        showSearching();
        listUpdate.start();

        // populateList(wikiDoc.getWikiWordMatchTermsWith(
        //         tfSearchWikiWord.getText().toString(), "", false));
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        guiHelper = new FragmentGuiHelper<WikiWordListActivity,
                WikiWordListFragment>(this);

        WikiDocument wikiDoc = WikiDocument.getActiveDocument();
        
        if (wikiDoc == null || savedInstanceState == null ||
                !savedInstanceState.containsKey("lastDatabaseChangeCount")) {
            lastSearch = null;
        }
        else {
            // First check if same wiki open yet
            if (wikiDoc.getWikiConfigPath().equals(savedInstanceState
                    .getString("wikiConfigPath"))) {
                lastSearch = savedInstanceState.getString("lastSearch");
                lastDatabaseChangeCount = savedInstanceState.getLong(
                        "lastDatabaseChangeCount");
                lastListState = savedInstanceState.getParcelable("listState");
            }
            else
                lastSearch = null;
        }
        
        listAdapter = new StringArrayListAdapter<MatchTermEntry>(guiHelper.getActivity(),
                R.layout.simple_list_item);
        
        listAdapter.setStringifier(new Stringifier<MatchTermEntry>() {
            public String toString(MatchTermEntry mte) {
                return mte.matchterm;
            }           
        });
        
        new RichMEListener(this).addEventKey("request clear lruWikiPaths")
                .setTarget(this, "clearLruWikiPaths").subscribeTo(
                WikiAndPadApplication.getMainEvent());

        setHasOptionsMenu(true);
    }

    
    @Override
    public View onCreateView (LayoutInflater inflater,
            ViewGroup container, Bundle savedInstanceState) {

        guiHelper.setContentView(R.layout.wiki_word_list, inflater,
                container, savedInstanceState);

        Button btnCreateWord = guiHelper.findViewById(R.id.btnCreateWord);
        tfSearchWikiWord = guiHelper.findViewById(
                R.id.tfSearchWikiWord);
        
        btnCreateWord.setOnClickListener(onBtnCreateWord);
        
        tfSearchWikiWord.addTextChangedListener(new TextWatcher() {
            public void onTextChanged(CharSequence s, int start, int before,
                    int count) {
            }
            
            public void beforeTextChanged(CharSequence s, int start, int count,
                    int after) {
            }
            
            public void afterTextChanged(Editable s) {
                updateList();
            }
        });
        
        guiHelper.getActivity().getWindow().setSoftInputMode(
                WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN);
        
        Button btnOpenWiki = guiHelper.findViewById(R.id.btnOpenWiki);
        btnOpenWiki.setOnClickListener(onBtnOpenWiki);

        guiHelper.setOnClickListener(R.id.btnClearSearch,
                "onCmdClearSearch");
        
        ListView lv = guiHelper.getListView();
        lv.setAdapter(listAdapter);
        
        registerForContextMenu(lv);

        return guiHelper.getContentView();
    }
    
    
    protected void updateShowHideUi() {
        Button btnOpenWiki = guiHelper.findViewById(R.id.btnOpenWiki);
        Button btnCreateWord = guiHelper.findViewById(R.id.btnCreateWord);

        WikiDocument wikiDoc = WikiDocument.getActiveDocument();
                
        if (wikiDoc == null) {
            // No wiki open
            btnOpenWiki.setVisibility(View.VISIBLE);
            btnCreateWord.setVisibility(View.GONE);
            tfSearchWikiWord.setVisibility(View.GONE);
        }
        else {
            // Wiki open
            btnOpenWiki.setVisibility(View.GONE);
            btnCreateWord.setVisibility(View.VISIBLE);
            tfSearchWikiWord.setVisibility(View.VISIBLE);
        }
    }
    
    
    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        
        WikiDocument wikiDoc = WikiDocument.getActiveDocument();
        if (wikiDoc == null)
            return;
        
        outState.putString("lastSearch", lastSearch);
        outState.putLong("lastDatabaseChangeCount", lastDatabaseChangeCount);
        outState.putString("wikiConfigPath", wikiDoc.getWikiConfigPath());
        outState.putParcelable("listState", getListView().onSaveInstanceState());
    }


    @Override
    public void onPause() {
        super.onPause();
        listUpdate.setThread(null);
        WikiAndPadApplication.getMainEvent().setEnabledListenersByKey(this, false);
    }


    @Override
    public void onResume() {
        super.onResume();
        WikiAndPadApplication.getMainEvent().setEnabledListenersByKey(this,
                true);
        
        updateShowHideUi(); 
        
        /*String WikiWordCcPAT = "(?:[" +
                StringOps.UPPERCASE +
                "]+[" +
                StringOps.LOWERCASE +
                "]+[" +
                StringOps.UPPERCASE +
                "]+[" +
                StringOps.LETTERS + StringOps.DIGITS +
                "]*|[" +
                StringOps.UPPERCASE +
                "]{2,}[" +
                StringOps.LOWERCASE +
                "]+)";

        Pattern re = java.util.regex.Pattern.compile("\\b(?<!~)" + WikiWordCcPAT + "\\b", 0);
        Matcher mt = re.matcher("hjdghf AutoCompletion");

        // Log.d("WikiAndPad", java.util.regex.Pattern.quote("["));
        // Log.d("WikiAndPad", Boolean.toString(mt.find()));
        // Log.d("WikiAndPad", mt.find()); */

        
        updateList();
    }
    
    /*
    public void onDetach() {
        Fragment f = getActivity().getSupportFragmentManager()
                .findFragmentByTag("baseTest");

        Log.d("WikiAndPad", "--Fragment.onDetach" + StringOps.toSaveString(f));
        
        if (f != null) {
            getActivity().getSupportFragmentManager().beginTransaction().remove(
                    f).commitAllowingStateLoss();
            getActivity().getSupportFragmentManager().executePendingTransactions();
        }
        super.onDetach();
    }
    */

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        inflater.inflate(R.menu.wiki_word_list, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }
    
    @Override
    public void onPrepareOptionsMenu(Menu menu) {
        MenuItem item = menu.findItem(R.id.mnCloseWiki);
        item.setVisible(WikiDocument.getActiveDocument() != null);

        item = menu.findItem(R.id.mnSearch);
        item.setVisible(WikiDocument.getActiveDocument() != null);
    }
    
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Intent intent;

        switch (item.getItemId()) {
        case R.id.mnOpenWiki:
            SharedPreferences prefs = WikiAndPadApplication.getAppPrefs();
            
            ArrayList<String> paths = StringOps.strToStrList(prefs.getString(
                    "lruWikiPaths", ""));

            FolderPicker mFileDialog = new FolderPicker(
                    guiHelper.getActivity(), onOpenWiki,
                    android.R.style.Theme, true, paths);
            mFileDialog.show();

            return true;
        case R.id.mnCloseWiki:
            try {
                WikiDocument.openDocument(null, true);
            }
            catch (FileNotFoundException e) {}
            catch (IOException e) {}
            catch (WikiDataException e) {}

            updateShowHideUi();
            updateList();
            return true;
        case R.id.mnPreferences:
            intent = new Intent(guiHelper.getActivity().getBaseContext(),
                    Preferences.class);
            startActivity(intent);
            
            return false;
        case R.id.mnSearch:
            guiHelper.getActivity().onSearchRequested();
            
            return false;
        case R.id.mnAbout:
            intent = new Intent(Intent.ACTION_VIEW);
            intent.setComponent(new ComponentName(guiHelper.getActivity(),
                    AboutActivity.class));

            startActivity(intent);
            
            return false;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    
    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        /*
        Intent intent = new Intent(Intent.ACTION_EDIT);
        intent.setComponent(new ComponentName(guiHelper.getActivity(), WikiWordEditActivity.class));
        intent.putExtra("wikiPageName", listContent.get(position).wikiPageName);
        
        startActivity(intent);
        */
        
        getMiscEvent().cloneIt().put("request openWikiWord")
                .put("wikiConfigPath",
                    WikiDocument.getActiveDocument().getWikiConfigPath())
                .put("wikiPageName", listAdapter.get(position).wikiPageName)
                .send();
    }
    
    
    public void onCmdClearSearch() {
        tfSearchWikiWord.setText("");
    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View v,
            ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        if (v == getListView()) {
            MenuInflater inflater = guiHelper.getActivity().getMenuInflater();
            inflater.inflate(R.menu.wiki_word_listitem_context, menu);
        }
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterContextMenuInfo info =
                (AdapterContextMenuInfo) item.getMenuInfo();
        MatchTermEntry mte;
        switch (item.getItemId()) {
        case R.id.mnRenameThis:
            mte = listAdapter.get(info.position);
            if (WikiDocument.getActiveDocument().getWikiName().equals(
                    mte.wikiPageName)) {
                guiHelper.alertInfo("Rename of root page not supported");
            }
            else {
                onRename(mte.wikiPageName);
            }
            
            return false;

        case R.id.mnDeleteThis:
            mte = listAdapter.get(info.position);
            if (WikiDocument.getActiveDocument().getWikiName().equals(
                    mte.wikiPageName)) {
                guiHelper.alertInfo("Delete of root page not possible");
            }
            else {
                onDelete(mte.wikiPageName);
            }

            return false;
        default:
            return super.onContextItemSelected(item);
        }
    }
    
    public void onRename(final String wikiPageName) {
    
        AlertDialog.Builder alert = new AlertDialog.Builder(guiHelper.getActivity());

        alert.setTitle("Rename " + wikiPageName);
        // alert.setMessage("Message");

        // Set an EditText view to get user input 
        final EditText input = new EditText(guiHelper.getActivity());
        input.setText(wikiPageName);
        input.setInputType(InputType.TYPE_CLASS_TEXT |InputType.TYPE_TEXT_VARIATION_NORMAL);
        alert.setView(input);

        alert.setPositiveButton("OK", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                String toWord = input.getText().toString();
                if (toWord.length() == 0) {
                    guiHelper.alertInfo("Can't rename to empty string");
                    dialog.dismiss();
                    return;
                }

                try {
                    WikiDocument.getActiveDocument().renameWikiWord(wikiPageName,
                            toWord, false);
                    updateList();
                } catch (DbWriteAccessException e) {
                    e.printStackTrace();
                    return;
                } catch (WikiDataException e) {
                    guiHelper.alertInfo("Can't rename to existing wikiword");
                    dialog.dismiss();
                    return;
                }
            }
        });

        alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                dialog.dismiss();
            }
        });

        alert.show();
    }

    public void onDelete(final String wikiPageName) {
        
        AlertDialog.Builder alert = new AlertDialog.Builder(guiHelper.getActivity());

        alert.setTitle("Delete " + wikiPageName);
        // alert.setMessage("Message");

        alert.setPositiveButton("OK", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                try {
                    WikiPage page = WikiDocument.getActiveDocument()
                            .getWikiPage(wikiPageName);
                    
                    page.deletePage(true);
                    updateList();
                } catch (DbWriteAccessException e) {
                    TheExceptionHandler.logException(e);
                    return;
                } catch (WikiDataException e) {
                    TheExceptionHandler.logException(e);
                    return;
                }
            }
        });

        alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                dialog.dismiss();
            }
        });

        alert.show();
    }
    
    public void clearLruWikiPaths() {
        SharedPreferences prefs = WikiAndPadApplication.getAppPrefs();
        
        Editor editor = prefs.edit();
        editor.putString("lruWikiPaths", "");
        editor.commit();
    }


    protected MiscEvent miscEvent = new MiscEvent(this);
    
    public MiscEvent getMiscEvent() {
        return miscEvent;
    }
    
}
