package de.mbutscher.wikiandpad.db;

import java.util.ArrayList;
import java.util.List;


public interface IWikiData {
    
    public abstract String getWikiConfigPath();

    public abstract void close();

    public abstract void beginSingleTransaction();

    public abstract void endSingleTransaction();

    public abstract void setSingleTransactionSuccessful();

    public abstract String getWikiPageContent(String wikiPageName)
            throws DbReadAccessException;

    /**
     * get the list of match terms with thisStr in them.
     * @param thisStr
     * @param orderBy
     * @param descend
     * @return
     */
    public abstract ArrayList<MatchTermEntry> getWikiWordMatchTermsWith(
            String thisStr, String orderBy, boolean descend);

    public abstract boolean isDefinedWikiPageName(String wikiPageName);



    /**
     * Sets the content, does not modify the cache information
        except cachedWikiPageLinkTermDict
     * @param wikiPageName
     * @param content
     * @throws DbWriteAccessException 
     */
    public abstract void setWikiPageContent(String wikiPageName,
            String content, double moddate, double creadate)
            throws DbWriteAccessException;

    public abstract void updateWikiWordMatchTerms(String wikiPageName,
            List<MatchTermEntry> wwmTerms, boolean syncUpdate);

    public abstract void renameWord(String wikiPageName, String toWord)
            throws DbWriteAccessException;

    /**
     * Return all unified names starting with startingWith (case sensitive)
     * @param startingWith
     * @return
     */
    public abstract List<String> getDataBlockUnifNamesStartingWith(
            String startingWith);

    /**
     * Return all unified names starting with startingWith (case sensitive)
     * and ending with endingWith
     * @param startingWith
     * @param endingWith
     * @return
     */
    public abstract List<String> getDataBlockUnifNamesStartingEndingWith(
            String startingWith, String endingWith);

    /**
     * Tries to rename datablock with name unifName to toUnifName.
     * If datablock unifName doesn't exist, nothing happens.
     * @param unifName
     * @param toUnifName
     * @throws DbWriteAccessException
     */
    public abstract void renameDataBlock(String unifName, String toUnifName)
            throws DbWriteAccessException;

    /** 
     * Delete data block with the associated unified name. If the unified name
        is not in database, nothing happens.
     * @param unifName
     */
    public abstract void deleteDataBlock(String unifName);

    
    /**
     * delete everything about the wikiword passed in. an exception is raised
        if you try and delete the wiki root node.
    
     * @param wikiPageName
     * @param delContent Should actual content be deleted as well? (Parameter is
                not part of official API)
     * @throws WikiDataException 
     */
    public abstract void deleteWikiPage(String wikiPageName, boolean delContent)
            throws WikiDataException;

    public abstract void deleteWikiPage(String wikiPageName)
            throws WikiDataException;

    public abstract void deleteTodos(String wikiPageName);

    public abstract void deleteAttributes(String wikiPageName);

    public abstract void deleteChildRelationships(String wikiPageName);

    public abstract String getWikiPageNameForLinkTerm(String linkTerm);

    /**
        Returns the name of the "first" wiki word. See getNextWikiPageName()
        for details. Returns either an existing wiki word or None if no
        wiki words in database.
        Function must work for read-only wiki.
     */
    public abstract String getFirstWikiPageName();

    /**
        Returns the "next" wiki word after currWord or None if no
        next word exists. If you begin with the first word returned
        by getFirstWikiPageName() and then use getNextWikiPageName() to
        go to the next word until no more words are available
        and if the list of existing wiki words is not modified during
        iteration, it is guaranteed that you have visited all real
        wiki words (no aliases) then.
        currWord  doesn't have to be an existing word itself.
        Function must work for read-only wiki.
    
     * @return
     */
    public abstract String getNextWikiPageName(String currWord);
    
    /**
     * @param wikiPageName
     * @param tt
     * @return Seconds since epoch
     */
    public abstract double getTimestamp(String wikiPageName, TimestampType tt);
    public abstract void setTimestamp(String wikiPageName, TimestampType tt,
            double secsSinceEpoch);
}