package de.mbutscher.wikiandpad.parsing;

import java.util.List;
import java.util.regex.Pattern;

import de.mbutscher.wikiandpad.utils.NotCurrentThreadException;

public class MatchFirst extends ParseExpression
        implements NecessaryRegexProvider {

    protected RegexCombiner regexCombiner;
    
    public MatchFirst(ParserElement[] exprs) {
        super(exprs);
        if (exprs.length > 0) {
            mayReturnEmpty = false;
            for(ParserElement e: exprs) {
                if (e.mayReturnEmpty) {
                    mayReturnEmpty = true;
                    break;
                }
            }
        }
        else
            mayReturnEmpty = true;
    }

    @Override
    protected ParsingResult parseImpl(String instring, int loc,
            ParsingState state, boolean doActions) throws ParseException,
            NotCurrentThreadException {
        
        int maxExcLoc = -1;
        ParseException maxException = null;

        if (regexCombiner == null) {
            for(ParserElement e: exprs) {
                try {
                    return e.parse(instring, loc, state, doActions);
                }
                catch (ParseException err) {
                    if (err.loc > maxExcLoc) {
                        maxException = err;
                        maxExcLoc = err.loc;
                    }
                }
                catch (ArrayIndexOutOfBoundsException err) {
                    if (instring.length() > maxExcLoc) {
                        maxException = new ParseException(instring,
                                instring.length(), e.errmsg, this);
                        maxExcLoc = instring.length();
                    }
                }
            }

            // only got here if no expression matched, raise exception for match
            // that made it the furthest

            if (maxException != null)
                throw maxException;
            else
                throw new ParseException(instring, loc,
                        String.format("No defined alternatives %s to match",
                        exprs), this);
        }
        else {
            RegexCombiner.FindAllResult findAll = regexCombiner.findAll(
                    instring, loc);
            
            List<Integer> styles = findAll.styles;
            loc = findAll.loc;
            
            
            if (findAll.styles.isEmpty()) {
                throw new ParseException(instring, findAll.loc, String.format(
                        "No defined alternatives %s to match",
                        exprs), this);
            }

            for (int st: styles) {
                try {
                    return exprs.get(st).parse(instring, loc, state, doActions);
                }
                catch (ParseException err) {
                    if (err.loc > maxExcLoc) {
                        maxException = err;
                        maxExcLoc = err.loc;
                    }
                }
                catch (ArrayIndexOutOfBoundsException err) {
                    if (instring.length() > maxExcLoc) {
                        maxException = new ParseException(instring,
                                instring.length(), err.getMessage(), this);
                        maxExcLoc = instring.length();
                    }
                }
            }
            
            // only got here if no expression matched, raise exception for
            // match that made it the furthest

            if (maxException != null) {
                throw maxException;
            }
            else {
                throw new ParseException(instring, loc, String.format(
                        "No defined alternatives %s to match",
                        exprs), this);
            }
        }
    }
    
    
    public RegexCombiner getRegexCombiner() {
        return regexCombiner;
    }

    public Pattern getRegex() {
        if (regexCombiner == null)
            return null;
        
        return Pattern.compile(regexCombiner.getCleanPattern(),
                regexCombiner.getFlags());
    }

    public int getRegexFlagsMask() {
        if (regexCombiner == null)
            return 0;

        return regexCombiner.getFlagsMask();
    }

    public boolean isRegexComplete() {
        return false;
    }
    
    @Override
    protected void _optimizeSub(OptimizerOptions options) {
        super._optimizeSub(options);

        if (regexCombiner != null || !options.regexcombine)
            return;

        RegexCombiner combiner = new RegexCombiner(exprs,
                RegexCombiner.REMODE_MATCH_ALL);
        if (combiner.combine())
            regexCombiner = combiner;
    }

}


