package de.mbutscher.wikiandpad.parsing;

import de.mbutscher.wikiandpad.utils.NotCurrentThreadException;

public class NotAny extends ParseElementEnhance {

    public NotAny(ParserElement expr) {
        super(expr);
        mayReturnEmpty = true;
    }


    @Override
    /**
     * Lookahead to disallow matching with the given parse expression.  NotAny
    does *not* advance the parsing position within the input string, it only
    verifies that the specified parse expression does *not* match at the current
    position.  Also, NotAny does *not* skip over leading whitespace. NotAny
    always returns a null token list.
     */
    protected ParsingResult parseImpl(String instring, int loc,
            ParsingState state, boolean doActions) throws ParseException,
            NotCurrentThreadException {

        boolean throwException = false;
        try {
            expr.tryParse(instring, loc, state);
            throwException = true;
        }
        catch (ParseException err) {}
        catch (ArrayIndexOutOfBoundsException err) {}
        
        if (throwException) {
            throw new ParseException("Found unwanted token " + expr, loc, null,
                    this);
        }

        return new ParsingResult(loc);
    }
    
    @Override
    public String toString() {
        return "~{" + expr + "}";
    }
}
