package de.mbutscher.wikiandpad.parsing;

import java.util.ArrayList;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import de.mbutscher.wikiandpad.utils.NotCurrentThreadException;


/**
 * Optional matching of the given expression.
       A default return string can also be specified, if the optional expression
       is not found.

 *
 */
public class Optional extends ParseElementEnhance {

    public Optional() {
        super(null);
    }

    public Optional(ParserElement expr) {
        super(expr);
    }
    
    @Override
    public boolean getNamedElementNeedsPacking() {
        return true;
    }
    

    public Pattern getRegex() {
        if (!(expr instanceof NecessaryRegexProvider))
            return null;
        
        if (buildingRegex)
            return null;
        
        buildingRegex = true;
        try {
            Pattern regex = ((NecessaryRegexProvider) expr).getRegex();
            if (regex == null)
                return null;
            try {
                return Pattern.compile("(?:" + regex.pattern() + ")?",
                        regex.flags());
            }
            catch (PatternSyntaxException e) {
                return null;
            }
        }
        finally {
            buildingRegex = false;
        }
    }

    public int getRegexFlagsMask() {
        if (!(expr instanceof NecessaryRegexProvider))
            return 0;
        
        if (buildingRegex)
            return -1;
        
        buildingRegex = true;
        try {
            return ((NecessaryRegexProvider) expr).getRegexFlagsMask();
        }
        finally {
            buildingRegex = false;
        }
    }

    public boolean isRegexComplete() {
        if (!(expr instanceof NecessaryRegexProvider))
            return false;
        
        if (buildingRegex)
            return false;

        buildingRegex = true;
        try {
            return ((NecessaryRegexProvider) expr).isRegexComplete();
        }
        finally {
            buildingRegex = false;
        }
    }

    
    @Override
    protected ParsingResult parseImpl(String instring, int loc,
            ParsingState state, boolean doActions) throws ParseException,
            NotCurrentThreadException {

        try {
            return expr.parseImpl(instring, loc, state,
                    doActions);
        }
        catch (ParseException e) {
            return new ParsingResult(loc, new ArrayList<SyntaxNode>(0));
        }
        catch (ArrayIndexOutOfBoundsException e) {
            return new ParsingResult(loc, new ArrayList<SyntaxNode>(0));
        }
    }
    
    @Override
    public String toString() {
        return "[" + expr.toString() + "]";
    }
}


/*




*/
            
