package de.mbutscher.wikiandpad.parsing;

import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;

/**
 * Abstract subclass of ParserElement, for combining and post-processing parsed tokens.
 *
 */
public abstract class ParseExpression extends ParserElement {

    protected List<ParserElement> exprs;

    public ParseExpression(ParserElement[] exprs) {
        if (exprs == null)
            this.exprs = null;
        else
            this.exprs = new ArrayList<ParserElement>(Arrays.asList(exprs));
    }
    
    public ParserElement get(int i) {
        return exprs.get(i);
    }
    
    @Override
    public boolean getNamedElementNeedsPacking() {
        return true;
    }
    
    public ParseExpression append(ParserElement pe) {
        exprs.add(pe);
        return this;
    }
    
    @Override
    public String toString() {
        return String.format("%s:(%s)", getClass().getName(), exprs);
    }
    
    @Override
    public ParserElement streamline() {
        super.streamline();
        
        for(ParserElement e: exprs) {
            e.streamline();
        }
        
        /*
        collapse nested And's of the form And( And( And( a,b), c), d) to And( a,b,c,d )
        but only if there are no parse actions or resultsNames on the nested And's
        (likewise for Or's and MatchFirst's)
        */
        ArrayList<ParserElement> newExprs = new ArrayList<ParserElement>();
        
        for(ParserElement other: exprs) {
            if (getClass().isInstance(other) &&
                    other.parseStartActions.isEmpty() &&
                    other.validateActions.isEmpty() &&
                    other.parseActions.isEmpty() &&
                    other.resultsName == null) {
                newExprs.addAll(((ParseExpression) other).exprs);
            }
            else {
                newExprs.add(other);
            }
        }
        exprs = newExprs;
        
        return this;
    }
    
    @Override
    protected void _optimizeSub(OptimizerOptions options) {
        // TODO Auto-generated method stub
        super._optimizeSub(options);
        ArrayList<ParserElement> newExprs = new ArrayList<ParserElement>(
                exprs.size());
        
        for (ParserElement exp: exprs) {
            newExprs.add(exp._realOptimize(options));
        }
        
        exprs = newExprs;
    }

}
