package de.mbutscher.wikiandpad.parsing;

import java.util.ArrayList;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import de.mbutscher.wikiandpad.utils.NotCurrentThreadException;

public class ZeroOrMore extends ParseElementEnhance
        implements NecessaryRegexProvider {

    public ZeroOrMore(ParserElement expr) {
        super(expr);
        mayReturnEmpty = true;
    }
    
    @Override
    public boolean getNamedElementNeedsPacking() {
        return true;
    }


    public Pattern getRegex() {
        if (!(expr instanceof NecessaryRegexProvider))
            return null;
        
        if (buildingRegex)
            return null;
        
        buildingRegex = true;
        try {
            Pattern regex = ((NecessaryRegexProvider) expr).getRegex();
            if (regex == null)
                return null;
            try {
                return Pattern.compile("(?:" + regex.pattern() + ")*",
                        regex.flags());
            }
            catch (PatternSyntaxException e) {
                return null;
            }
        }
        finally {
            buildingRegex = false;
        }
    }

    public int getRegexFlagsMask() {
        if (!(expr instanceof NecessaryRegexProvider))
            return 0;
        
        if (buildingRegex)
            return -1;
        
        buildingRegex = true;
        try {
            return ((NecessaryRegexProvider) expr).getRegexFlagsMask();
        }
        finally {
            buildingRegex = false;
        }
    }

    public boolean isRegexComplete() {
        if (!(expr instanceof NecessaryRegexProvider))
            return false;
        
        if (buildingRegex)
            return false;

        buildingRegex = true;
        try {
            return ((NecessaryRegexProvider) expr).isRegexComplete();
        }
        finally {
            buildingRegex = false;
        }
    }

    
    @Override
    protected ParsingResult parseImpl(String instring, int loc,
            ParsingState state, boolean doActions) throws ParseException,
            NotCurrentThreadException {

        ParsingResult parsingResult = new ParsingResult(loc);
        ArrayList<SyntaxNode> tokens = new ArrayList<SyntaxNode>();
        
        try {
            parsingResult = expr.parse(instring, loc, state, doActions);
            tokens.addAll(parsingResult.retTokens);
            while (true) {
                parsingResult = expr.parse(instring, parsingResult.nextLoc, state,
                        doActions);
                tokens.addAll(parsingResult.retTokens);
            }
        }
        catch (ParseException err) {}
        catch (ArrayIndexOutOfBoundsException err) {}
        
        return new ParsingResult(parsingResult.nextLoc, tokens);
    }
    
    @Override
    public String toString() {
        return "[" + expr + "]...";
    }
}
