package de.mbutscher.wikiandpad.utils;

import java.util.Iterator;

/**
 * Wraps an {@link Iterator<T>} and returns elements based on return of
 * {@link #filter(Object)}
 * 
 * @author Michael Butscher
 *
 * @param <T>
 */
public abstract class FilteredRoIterator<T> implements IterableIterator<T> {

    protected T nextResult=null;
    protected boolean _hasNext=false;
    protected Iterator<T> baseIterator=null;
    

    public FilteredRoIterator() {
    }

    public FilteredRoIterator(Iterator<T> base) {
        init(base);
    }

    public void init(Iterator<T> base) {
        this.baseIterator = base;
        next();
    }
    
    /**
     * Returns true iff result should be returned by the iterator.
     */
    public abstract boolean filter(T result);
    
    public boolean hasNext() {
        return _hasNext;
    }

    public T next() {
        T result = nextResult;
        _hasNext = false;
        while(baseIterator.hasNext())
        {
            T item=baseIterator.next();
            if (filter(item)) {
                nextResult = item;
                _hasNext = true;
                break;
            }
        }
        
        return result;
    }

    public void remove() {
        throw new UnsupportedOperationException();
    }
    
    public Iterator<T> iterator() {
        return this;
    }

}
